-- Settings are now modified under Interface Addon panel in game.

-- Default options that are loaded the first time the mod is loaded in the game
CEPetBar_Options = {};

--------------------------------------------------------------------------------
-- Local variables

-- variables to store the original Pet name color
local petOrigR, petOrigG, petOrigB = nil;

-- casting related variables
CEPB_PetCasting = false;
CEPB_PetChanneling = false;
local petSpell, petStartTime, petEndTime, petFont, petOldFontSize, petOtherStuff = nil;

local IsPlayerInVehicle_Pet = false;
local didPetGoPoof = false;

--------------------------------------------------------------------------------
-- Table management functions

function GetPetVal(option, table)
	if (table) then
		return CEPetBar_Options[table][option];
	else
		return CEPetBar_Options[option];
	end
end

function SetPetVal(option, value, table)
	if (table) then
		CEPetBar_Options[table][option] = value;
	else
		CEPetBar_Options[option] = value;
	end
	
	-- Update the global table
	_G["CEPetBar_Options"] = CEPetBar_Options;
end

function GetPetLength()
  local count = 0;
  for _ in pairs(CEPetBar_Options) do count = count + 1 end
  return count;
end

--------------------------------------------------------------------------------
-- Event / Texture Registrations

function CEPetBar_SetPetTexture()
	local texture = PetFrameTexture:GetTexture();
	if (texture ~= "Interface\\AddOns\\CEPlayerBar\\textures\\PetReplacementFrame") then
		CEPB_OldPetTexture = texture;
	end
	PetFrameTexture:SetTexture("Interface\\AddOns\\CEPlayerBar\\textures\\PetReplacementFrame.tga");
	PetName:SetFont(petFont, GetPetVal("FontSize"), petOtherStuff);
end

function CEPetBar_UnloadPetTexture()
	PetFrameTexture:SetTexture(CEPB_OldPetTexture);
	PetName:SetFont(petFont, petOldFontSize, petOtherStuff);
end

function CEPetBar_RegisterPetEvents(self, excludeRequired)
	if (not excludeRequired) then
		self:RegisterEvent("UNIT_PET");
		self:RegisterEvent("UNIT_PET_EXPERIENCE");
	end

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("VARIABLES_LOADED");

	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_START");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_STOP");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	CEPetBar_SetPetTexture();
end

function CEPetBar_UnregisterPetEvents(self, excludeRequired)
	if (not excludeRequired) then
		self:UnregisterEvent("UNIT_PET");
		self:UnregisterEvent("UNIT_PET_EXPERIENCE");
	end

	self:UnregisterEvent("PLAYER_ENTERING_WORLD");
	self:UnregisterEvent("VARIABLES_LOADED");

	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	self:UnregisterEvent("UNIT_SPELLCAST_START");
	self:UnregisterEvent("UNIT_SPELLCAST_STOP");
	self:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
end

--------------------------------------------------------------------------------
-- State management functions

function CEPetBar_Load()
	CEPetBar_RegisterPetEvents(_G["CEPetBar_Frame"], false);
	CEPetBar:Show();
	CEPetBar_ShowXP();
	if (GetPetVal("PercentEnabled")) then
		CEPetBarPercent:Show();
		CEPetBarPercent:SetText(CEPetBar_Process(GetPetVal("PercentPattern")));	
	end
end

function CEPetBar_Unload()
	CEPetBar_UnregisterPetEvents(_G["CEPetBar_Frame"], false);
	CEPetBar_UnloadPetTexture();
	CEPetBar:Hide();
	CEPetBarPercent:Hide();
				
end

function CEPetBar_ManageCurrentState()
	local isPetBarEnabled = GetPlayerVal("DisplayPetBar");
	local isCastEnabled = GetPetVal("DisplayCast");
	local isExpEnabled = GetPetVal("DisplayExp");
	if (isPetBarEnabled and (isCastEnabled or isExpEnabled)) then
		CEPetBar_Load();
	else
		CEPetBar_Unload();
	end
end

--------------------------------------------------------------------------------
-- Load

function CEPetBar_SetDefaults()
	petOrigR, petOrigG, petOrigB = PetName:GetTextColor();
	petFont, petOldFontSize, petOtherStuff = PetName:GetFont();

	if (GetPlayerVal("DisplayPetBar")) then
		CEPetBar_RegisterPetEvents(_G["CEPetBar_Frame"], false);
		CEPB_PetCasting = false;
	end
end

function CEPetBar_SetVariables()
	if ( GetPetLength() == 0 ) then
		CEPetBar_Options = _G["CEPetBar_Options"];
	end
	if ( GetPetLength() == 0 ) then
		CEPetBar_Options = {
			["ChannelingBar"] = {
				["R"] = 1.0,
				["G"] = 0.7,
				["B"] = 0.0,
				["O"] = 1.0
			},
			["CastingBar"] = {
				["R"] = 1.0,
				["G"] = 0.7,
				["B"] = 0.0,
				["O"] = 1.0
			},
			["CastingText"] = {
				["R"] = 1.0,
				["G"] = 0.82,
				["B"] = 0.0
			},
			["Exp"] = {
				["R"] = 1.0,
				["G"] = 1.0,
				["B"] = 0.2,
				["O"] = 1.0
			},
			["DisplayTime"] = true,
			["DisplayExp"] = true,
			["DisplayCast"] = true,
			["ShowSpellName"] = true,
			["FontSize"] = 8,
			["HoverPattern"] = "&percentxp% XP",
			["NoHoverPattern"] = "&Petname",
			["PercentEnabled"] = false,
			["PercentPattern"] = "&remainingxppercent%",
			["Version"] = 1.1
		}
	end 

	if (not didInit) then
		petOrigR, petOrigG, petOrigB = PetName:GetTextColor();
		petFont, petOldFontSize, petOtherStuff = PetName:GetFont();

		if (GetPlayerVal("DisplayPetBar")) then
			CEPetBar_RegisterPetEvents(_G["CEPetBar_Frame"], false);
			CEPB_PetCasting = false;
		end
		didInit = true;
	end
end

--------------------------------------------------------------------------------
-- Event handling

function CEPetBar_OnEvent (self, event, ...)
	IsPlayerInVehicle_Pet = false;
	if (UnitInVehicle("player")) then
		IsPlayerInVehicle_Pet = true;
	end
	
	if (GetPlayerVal("DisplayPetBar") and not IsPlayerInVehicle_Pet) then

		local isEligible = false;
		local type, _, _, _, _, _, _, _, _ = ...;

		-- Check vehicle status
		if (type == "pet") then
			isEligible = true;
		end

		if ( event == "UNIT_SPELLCAST_START" and isEligible and GetPetVal("DisplayCast") ) then
			-- Get spell information
			petSpell, _, _, _, petStartTime, petEndTime = UnitCastingInfo(type);
			petStartTime = petStartTime / 1000;
			petEndTime = petEndTime / 1000;

			-- Set cast bar properties
			CEPetBar:SetStatusBarColor(GetPetVal("R", "CastingBar"), GetPetVal("G", "CastingBar"), GetPetVal("B", "CastingBar"), GetPetVal("O", "CastingBar"));
			CEPetBar:SetMinMaxValues(petStartTime, petEndTime);
			CEPetBar:SetValue(petStartTime);

			-- Set spell text properties
			PetName:SetFont(petFont, GetPetVal("FontSize"), petOtherStuff);
			PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));

			-- Show spell information
			if (GetPetVal("DisplayTime") and GetPetVal("ShowSpellName")) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText( petSpell .. " " .. ((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText( petSpell .. " " .. (petEndTime - petStartTime) / 1000);
				end

			elseif (GetPetVal("DisplayTime") and not GetPetVal("ShowSpellName")) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText(((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText((petEndTime - petStartTime) / 1000);
				end

			elseif (not GetPetVal("DisplayTime") and GetPetVal("ShowSpellName")) then
				PetName:SetText( petSpell );
			end

			CEPB_PetCasting = true;

		elseif ( event == "UNIT_SPELLCAST_STOP" and isEligible and GetPetVal("DisplayCast") ) then
			if (CEPB_PetCasting) then
				CEPB_PetCasting = false;
				PetName:SetFont(petFont, petOldFontSize, petOtherStuff);
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
				return;
			end

		elseif (event == "UNIT_SPELLCAST_CHANNEL_STOP" and isEligible and GetPetVal("DisplayCast") ) then
			if (CEPB_PetChanneling) then
				CEPB_PetChanneling = false;
				PetName:SetFont(petFont, petOldFontSize, petOtherStuff);
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
				return;
			end

		elseif ( (event == "UNIT_SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED") and isEligible and GetPetVal("DisplayCast") ) then
			CEPB_PetCasting = false;
			CEPB_PetChanneling = false;
			CEPetBar_ShowXP();
			CEPetBar_RealUpdate();
			return;

		elseif ( event == "UNIT_SPELLCAST_DELAYED"  and isEligible and GetPetVal("DisplayCast") ) then
			local _, _, _, _, _, newEndTime = UnitCastingInfo(type);
			newEndTime = newEndTime / 1000;
			petEndTime = newEndTime;
			CEPetBar:SetMinMaxValues(petStartTime, petEndTime);

			if (GetPetVal("DisplayTime")) then
				if (math.floor(math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) == math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) then
					PetName:SetText((math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) .. ".0");
				else
					PetName:SetText(math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10);
				end
			end

		elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" and isEligible and GetPetVal("DisplayCast") ) then
			-- Get spell information
			petSpell, _, _, _, petStartTime, petEndTime = UnitChannelInfo(type);
			petStartTime = petStartTime / 1000;
			petEndTime = petEndTime / 1000;

			-- Set cast bar properties
			CEPetBar:SetMinMaxValues(petStartTime, petEndTime);
			CEPetBar:SetValue(petEndTime);
			CEPetBar:SetStatusBarColor(GetPetVal("R", "ChannelingBar"), GetPetVal("G", "ChannelingBar"), GetPetVal("B", "ChannelingBar"), GetPetVal("O", "ChannelingBar"));

			-- Set spell text properties
			PetName:SetFont(petFont, GetPetVal("FontSize"), petOtherStuff);
			PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));

			-- Show spell information
			if (GetPetVal("DisplayTime") and GetPetVal("ShowSpellName")) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText( petSpell .. " " .. ((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText( petSpell .. " " .. (petEndTime - petStartTime) / 1000);
				end

			elseif (GetPetVal("DisplayTime") and not GetPetVal("ShowSpellName")) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText(((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText((petEndTime - petStartTime) / 1000);
				end

			elseif (not GetPetVal("DisplayTime") and GetPetVal("ShowSpellName")) then
				PetName:SetText( petSpell );
			end

			CEPB_PetCasting = false;
			CEPB_PetChanneling = true;

		elseif ( event == "UNIT_SPELLCAST_CHANNEL_UPDATE" and isEligible and GetPetVal("DisplayCast") ) then
			local _, _, _, _, _, newEndTime = UnitChannelInfo(type);
			if (newEndTime ~= nil) then
				newEndTime = newEndTime / 1000;
				if ( GetTime() >= newEndTime ) then
					CEPB_PetChanneling = false;
					CEPetBar_ShowXP();
					CEPetBar_RealUpdate();
					return;

				elseif ( CEPB_PetChanneling ) then
					local durationShift = petEndTime - newEndTime;
					petEndTime = newEndTime;
					CEPetBar:SetValue( CEPetBar:GetValue() - durationShift );
					if (GetPetVal("DisplayTime")) then
						if (math.floor(math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) == math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) then
							PetName:SetText( petSpell .. " " .. (math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) .. ".0");
						else
							PetName:SetText( petSpell .. " " .. math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10);
						end
					end
				end
			end

		elseif (event == "UNIT_PET_EXPERIENCE" and not CEPB_PetCasting and not CEPB_PetChanneling) then
			CEPetBar_ShowXP();
			CEPetBar_RealUpdate();
			CEPB_PetCasting = 0;

		elseif (event == "UNIT_PET" and not CEPB_PetCasting and not CEPB_PetChanneling) then
			CEPB_PetCasting = false;

			local isItAlive = false;
			if (UnitExists("pet")) then
				isItAlive = true;
			end

			if (isItAlive or not IsPlayerInVehicle_Pet) then
				didPetGoPoof = false;
			elseif (not isItAlive or IsPlayerInVehicle_Pet) then
				didPetGoPoof = true;
			end
			
			if (not didPetGoPoof) then
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
				CEPetBar_RegisterPetEvents(self, true);
			else
				CEPetBar_UnregisterPetEvents(self, true);
			end
		end
	end
end

function CEPetBar_ProcessBarSettingsWhileCasting()
	if (CEPB_PetChanneling) then
		CEPetBar:SetStatusBarColor(GetPetVal("R", "ChannelingBar"), GetPetVal("G", "ChannelingBar"), GetPetVal("B", "ChannelingBar"), GetPetVal("O", "ChannelingBar"));
		PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));
	elseif (CEPB_PetCasting) then
		CEPetBar:SetStatusBarColor(GetPetVal("R", "CastingBar"), GetPetVal("G", "CastingBar"), GetPetVal("B", "CastingBar"), GetPetVal("O", "CastingBar"));
		PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));
	end
end

--------------------------------------------------------------------------------
-- Handle clicking on new bar (extend pet bar basically)

function CEPetBar_OnClick(self, button)
	if ( SpellIsTargeting() and button == "RightButton" ) then
		SpellStopTargeting();
		return;
	end

	if ( button == "LeftButton" ) then
		if ( SpellIsTargeting() ) then
			-- SpellTargetUnit("pet");
		elseif ( CursorHasItem() ) then
			AutoEquipCursorItem();
		else
			-- TargetUnit("pet");
		end
	else
		ToggleDropDownMenu(1, nil, PetFrameDropDown, "PetFrame", 106, 27);
		return;
	end
end

--------------------------------------------------------------------------------
-- Tooltip

function CEPetBar_ProcessToolTip()
	if (CEPetBar_IsHovering and (CEPB_PetChanneling or CEPB_PetCasting)) then
		GameTooltip:SetOwner(PlayerFrame, "ANCHOR_CURSOR");
		GameTooltip:ClearLines();
		GameTooltip:AddLine(PetName:GetText());
		GameTooltip:Show();
	end
end

--------------------------------------------------------------------------------
-- Bar state functions

function CEPetBar_Update(self)
	IsPlayerInVehicle_Pet = false;
	if (UnitInVehicle("player")) then
		IsPlayerInVehicle_Pet = true;
	end

	CEPetBar_RealUpdate();

	if (not CEPetBar_IsHovering) then
		GameTooltip:Hide();
	end
end

function CEPetBar_RealUpdate()
	if (GetPlayerVal("DisplayPetBar") and not IsPlayerInVehicle_Pet) then
		if (UnitExists("pet")) then
			if (not CEPB_PetCasting) then
				if (CEPetBar_IsHovering) then
					PetName:SetText(CEPetBar_Process(GetPetVal("HoverPattern")));
				else
					local tempVal = false;
					if (UnitInVehicle("player")) then
						tempVal = true;
					end

					if (not tempVal) then
						PetName:SetText(CEPetBar_Process(GetPetVal("NoHoverPattern")));
					end
				end

				if (GetPetVal("PercentEnabled")) then
					CEPetBarPercent:Show();
					CEPetBarPercent:SetText(CEPetBar_Process(GetPetVal("PercentPattern")));
				else
					CEPetBarPercent:Hide();
				end

				if (GetPetVal("DisplayTime")) then
					PetName:SetTextColor(petOrigR, petOrigG, petOrigB);
				end
			end

			CEPetBar_SetPetTexture();
		end
	end
end

function CEPetBar_ShowXP()
	local currentXP, maxXP = GetPetExperience();
	if (maxXP == 0) then
		maxXP = 1;
		currentXP = 0;
	end
	if (GetPlayerVal("DisplayPetBar") and GetPetVal("DisplayExp") and not IsPlayerInVehicle_Pet) then
		CEPetBar:SetMinMaxValues(min(0, currentXP), maxXP);
		CEPetBar:SetValue(currentXP);
		CEPetBar:SetStatusBarColor(GetPetVal("R", "Exp"), GetPetVal("G", "Exp"), GetPetVal("B", "Exp"), GetPetVal("O", "Exp"));
	elseif (GetPlayerVal("DisplayPetBar") and not GetPetVal("DisplayExp") and not IsPlayerInVehicle_Pet) then
		CEPetBar:SetMinMaxValues(min(0, currentXP), maxXP);
		CEPetBar:SetValue(0);
	end
end

function CEPetBar_UpdateTimers(self)
	if (GetPlayerVal("DisplayPetBar") and not IsPlayerInVehicle_Pet) then
		if (CEPB_PetCasting or CEPB_PetChanneling) then
			local time = GetTime();
			if ( time  > petEndTime ) then
				time  = petEndTime;
			end

			if ( CEPB_PetCasting ) then
				CEPetBar:SetValue(time);
			elseif ( CEPB_PetChanneling ) then
				local barValue = petStartTime + (petEndTime - time);
				CEPetBar:SetValue(barValue);
			end

			if (GetPetVal("DisplayTime") and GetPetVal("ShowSpellName")) then
				if (math.floor(math.floor((petEndTime * 10) - (time * 10))/10) == math.floor((petEndTime * 10) - (time * 10))/10) then
					PetName:SetText( petSpell .. " " .. (math.floor((petEndTime * 10) - (time * 10))/10)..".0");
				else
					PetName:SetText( petSpell .. " " .. math.floor((petEndTime * 10) - (time * 10))/10);
				end

			elseif (GetPetVal("DisplayTime") and not GetPetVal("ShowSpellName")) then
				if (math.floor(math.floor((petEndTime * 10) - (time * 10))/10) == math.floor((petEndTime * 10) - (time * 10))/10) then
					PetName:SetText((math.floor((petEndTime * 10) - (time * 10))/10)..".0");
				else
					PetName:SetText(math.floor((petEndTime * 10) - (time * 10))/10);
				end

			elseif (not GetPetVal("DisplayTime") and GetPetVal("ShowSpellName")) then
				PetName:SetText( petSpell );
			end
		end
	end
	
	-- I set the texture here because when the Interface Addons page is closed, the texture resets
	-- I spent over an hour trying to find out what did it without luck.  I tried to use the
	-- UNIT_POWER event which triggers every 2 seconds but that isn't triggered when the bar is full.
	-- As a debugging note, RegisterAllEvents showed no event being raised when the Addon page is
	-- closed so I'd have to isolate the function calling it and securely hook it to directly
	-- address the problem.
	if (GetPlayerVal("DisplayPetBar") and IsPlayerInVehicle_Pet == 0 and UnitExists("pet")) then
		CEPetBar_SetPetTexture();
	end

	CEPetBar_ProcessToolTip();
end

--------------------------------------------------------------------------------
-- Process variables

function CEPetBar_Process(val)
	local currentXP, maxXP = GetPetExperience();
	local remainingXpPer;

	if (maxXP == 0) then
		remainingXpPer = 0;
	else
		remainingXpPer = floor((maxXP - currentXP)/maxXP*100);
	end

	if (maxXP == 0) then
		currentXP = 0;
		maxXP = 1;
	end

	val = string.gsub(val, "&Petname", UnitName("pet"));
	val = string.gsub(val, "&percentxp", floor(currentXP/maxXP*100));
	val = string.gsub(val, "&remainingxppercent", remainingXpPer);
	val = string.gsub(val, "&currentxp", currentXP);
	val = string.gsub(val, "&maxxp", maxXP);
	val = string.gsub(val, "&remainingxp", maxXP-currentXP);
	return val;
end