-- Handles the storage / populating of fields in the UI

local _G = _G;
local pairs = pairs;

--+++++++++++++++++++++++++++++++
-- Custom Drop Down Trigger
--+++++++++++++++++++++++++++++++

function CEPB:CustomDropDown_OnClick(ddl, loadValue)
	local valueToPass;
	if (loadValue) then 
		valueToPass = loadValue;
	else 
		valueToPass = self.value; 
	end

	if (not loadValue) then
		UIDropDownMenu_SetSelectedValue(ddl, valueToPass);
	end

	if (ddl.CEPBVar == "PlayerFormulaLookups") then 
		CEPB:LoadPlayer_FormulaLookups(valueToPass); 
	elseif (ddl.CEPBVar == "PlayerActionPatterns") then 
		CEPB:LoadPlayer_ActionPatterns(valueToPass); 
	elseif (ddl.CEPBVar == "PlayerBarSettings") then 
		CEPB:LoadPlayer_BarSettings(valueToPass); 
	elseif (ddl.CEPBVar == "PetFormulaLookups") then 
		CEPB:LoadPet_FormulaLookups(valueToPass); 
	elseif (ddl.CEPBVar == "PetActionPatterns") then 
		CEPB:LoadPet_ActionPatterns(valueToPass); 
	elseif (ddl.CEPBVar == "PetBarSettings") then 
		CEPB:LoadPet_BarSettings(valueToPass);
	end
end

--+++++++++++++++++++++++++++++++
-- Load Value Functions
--+++++++++++++++++++++++++++++++

function CEPB:LoadPlayer_FormulaLookups(value)
	-- The lookup dropdown
	-- Contains static values
	local editBox = _G["CEPBOptionsFrame_EditBox2_EditBox"];
	
	if (value == "A_Default") then
		editBox:SetText("");
	elseif (value == "Name") then
		editBox:SetText("&playername");
	elseif (value == "CurrentExp") then
		editBox:SetText("&currentxp");
	elseif (value == "MaxExp") then
		editBox:SetText("&maxxp");
	elseif (value == "PercentExp") then
		editBox:SetText("&percentxp");
	elseif (value == "RestExp") then
		editBox:SetText("&restxp");
	elseif (value == "RemainingExp") then
		editBox:SetText("&remainingxp");
	elseif (value == "PercentRemainingExp") then
		editBox:SetText("&remainingxppercent");
	end
	
	editBox:SetFocus();
	editBox:HighlightText();
end

-------------------------------------------------

function CEPB:LoadPlayer_ActionPatterns(value)
	-- The pattern dropdown
	local editBox = _G["CEPBOptionsFrame_EditBox1_EditBox"];
	local editContainer = _G["CEPBOptionsFrame_EditBox1"];
	editBox.CEPBType = "player"; -- in case it isn't already set
	editBox.CEPBVar = value;
	editBox.IsLoad = true;

	if (value == "A_Default") then
		editBox:SetText("");
		editContainer:Hide();
	else
		editContainer:Show();
		editBox:SetText(GetPlayerVal(value));
	end
end

-------------------------------------------------

function CEPB:LoadPlayer_BarSettings(value)
	-- The bar customization dropdown
	local shouldSetColor = false;
	local opacitySlider = _G["CEPBOptionsFrame_Slider2"];
	local opacitySliderValue = _G["CEPBOptionsFrame_Slider2Slider"];
	local colorPicker = _G["CEPBOptionsFrame_Color1"];
	local swatch = _G[colorPicker:GetName().."NormalTexture"];
	colorPicker.hasOpacity = false;
	
	-- Overwrite the opacitySlider's values so we can update the saved variables
	opacitySlider.CEPBVar = "O";
	opacitySlider.CEPBTable = value;
	opacitySliderValue.CEPBVar = "O";
	opacitySliderValue.CEPBTable = value;

	if (value == "A_Default") then
		opacitySlider:Hide();
		colorPicker:Hide();
	elseif (value == "CastingText") then
		-- Only one that doesn't have opacity
		opacitySlider:Hide();
		colorPicker:Show();
		shouldSetColor = true;
	else
		opacitySlider:Show();
		colorPicker:Show();
		shouldSetColor = true;
		opacitySlider.isLoading = true;
		opacitySliderValue:SetValue(GetPlayerVal("O", value) * 100);
		opacitySlider.isLoading = false;
	end
	
	if (shouldSetColor) then
		colorPicker.r = GetPlayerVal("R", value);
		colorPicker.g = GetPlayerVal("G", value);
		colorPicker.b = GetPlayerVal("B", value);
		colorPicker.CEPBVal = value;
	    swatch:SetVertexColor(colorPicker.r, colorPicker.g, colorPicker.b);
	end
end

-------------------------------------------------

function CEPB:LoadPet_FormulaLookups(value)
	-- The lookup dropdown
	-- Contains static values
	local editBox = _G["CEPBOptionsFrame_EditBox4_EditBox"];
	
	if (value == "A_Default") then
		editBox:SetText("");
	elseif (value == "Name") then
		editBox:SetText("&Petname");
	elseif (value == "CurrentExp") then
		editBox:SetText("&currentxp");
	elseif (value == "MaxExp") then
		editBox:SetText("&maxxp");
	elseif (value == "PercentExp") then
		editBox:SetText("&percentxp");
	elseif (value == "RemainingExp") then
		editBox:SetText("&remainingxp");
	elseif (value == "PercentRemainingExp") then
		editBox:SetText("&remainingxppercent");
	end
	
	editBox:SetFocus();
	editBox:HighlightText();
end

-------------------------------------------------

function CEPB:LoadPet_ActionPatterns(value)
	-- The pattern dropdown
	local editBox = _G["CEPBOptionsFrame_EditBox3_EditBox"];
	local editContainer = _G["CEPBOptionsFrame_EditBox1"];
	editBox.CEPBType = "pet"; -- in case it isn't already set
	editBox.CEPBVar = value;
	editBox.IsLoad = true;

	if (value == "A_Default") then
		editBox:SetText("");
		editContainer:Hide();
	else
		editContainer:Show();
		editBox:SetText(GetPetVal(value));	
	end
end

-------------------------------------------------

function CEPB:LoadPet_BarSettings(value)
	-- The bar customization dropdown
	local shouldSetColor = false;
	local opacitySlider = _G["CEPBOptionsFrame_Slider4"];
	local opacitySliderValue = _G["CEPBOptionsFrame_Slider4Slider"];
	local colorPicker = _G["CEPBOptionsFrame_Color2"];
	local swatch = _G[colorPicker:GetName().."NormalTexture"];
	colorPicker.hasOpacity = false;

	-- Overwrite the opacitySlider's values so we can update the saved variables
	opacitySlider.CEPBVar = "O";
	opacitySlider.CEPBTable = value;
	opacitySliderValue.CEPBVar = "O";
	opacitySliderValue.CEPBTable = value;

	if (value == "A_Default") then
		opacitySlider:Hide();
		colorPicker:Hide();
	elseif (value == "CastingText") then
		-- Only one that doesn't have opacity
		opacitySlider:Hide();
		colorPicker:Show();
		shouldSetColor = true;
	else
		opacitySlider:Show();
		colorPicker:Show();
		shouldSetColor = true;
		opacitySlider.isLoading = true;
		opacitySliderValue:SetValue(GetPetVal("O", value) * 100);
		opacitySlider.isLoading = false;
	end
	
	if (shouldSetColor) then
		colorPicker.r = GetPetVal("R", value);
		colorPicker.g = GetPetVal("G", value);
		colorPicker.b = GetPetVal("B", value);
		colorPicker.CEPBVal = value;
	    swatch:SetVertexColor(colorPicker.r, colorPicker.g, colorPicker.b);
	end
end

--+++++++++++++++++++++++++++++++
-- Set Value Functions
--+++++++++++++++++++++++++++++++

function CEPB:OptionsCheckButton_OnClick(chk)
	--Stores the checkbox values
    local isChecked = false;
    if (chk:GetChecked()) then
        isChecked = true;
    end
    
	if (chk.CEPBType == "player") then
		SetPlayerVal(chk.CEPBVar, isChecked);
		
		-- Process new setting
		if (chk.CEPBVar == "PercentEnabled") then
			CEPlayerBar_RealUpdate();
		elseif (chk.CEPBVar == "DisplayPetBar") then
			if (isChecked) then
				CEPetBar_Load();
			else
				CEPetBar_Unload();
			end
		elseif (chk.CEPBVar == "DisplayExp" or chk.CEPBVar == "DisplayCast") then
			CEPlayerBar_ManageCurrentState();
		end
		
	elseif (chk.CEPBType == "pet") then
		SetPetVal(chk.CEPBVar, isChecked);
		
		-- Process new setting
		if (chk.CEPBVar == "PercentEnabled") then
			CEPetBar_RealUpdate();
		elseif (chk.CEPBVar == "DisplayExp" or chk.CEPBVar == "DisplayCast") then
			CEPetBar_ManageCurrentState();
		end
		
	end
end

-------------------------------------------------

function CEPB:OptionsEditBox_OnValueChanged(editBox)
	if (editBox.IsLoad) then
		editBox.IsLoad = false;
	else
		if (editBox:GetName() == "CEPBOptionsFrame_EditBox1_EditBox" or editBox:GetName() == "CEPBOptionsFrame_EditBox3_EditBox" and editBox.CEPBVar ~= "A_Default") then
			if (editBox.CEPBType == "player") then
				SetPlayerVal(editBox.CEPBVar, editBox:GetText());
				if (editBox.CEPBVar == "NoHoverPattern") then
					CEPlayerBar_RealUpdate();
				end
			elseif (editBox.CEPBType == "pet") then
				SetPetVal(editBox.CEPBVar, editBox:GetText());
				if (editBox.CEPBVar == "NoHoverPattern") then
					CEPetBar_RealUpdate();
				end
			end
		end
	end
end

-------------------------------------------------

function CEPB:OptionsSlider_OnValueChanged(slider, value)
	-- Stores the slider values
	-- The slider in this case is the actual bar, and not the whole thing.
	if (not slider.isLoading) then
		if (slider.CEPBType == "player") then
			if (slider.CEPBTable) then
				SetPlayerVal(slider.CEPBVar, slider:GetValue() / 100, slider.CEPBTable);
				if (CEPB_PlayerCasting or CEPB_PlayerChanneling) then
					CEPlayerBar_ProcessBarSettingsWhileCasting();
				else
					CEPlayerBar_ShowXP();
					CEPlayerBar_RealUpdate();
				end
			else
				SetPlayerVal(slider.CEPBVar, slider:GetValue());
			end
		elseif (slider.CEPBType == "pet") then
			if (slider.CEPBTable) then
				SetPetVal(slider.CEPBVar, slider:GetValue() / 100, slider.CEPBTable);
				if (CEPB_PetCasting or CEPB_PetChanneling) then
					CEPetBar_ProcessBarSettingsWhileCasting();
				else
					CEPetBar_ShowXP();
					CEPetBar_RealUpdate();
				end
			else
				SetPetVal(slider.CEPBVar, slider:GetValue());
			end
		end
	end
end

-------------------------------------------------

function CEPB:OptionsFrame_SetColor(f,s,k)
	-- Sets the colors of the config from a color swatch
	local r,g,b = ColorPickerFrame:GetColorRGB();
	local swatch = _G[s];
	local frame = _G[f];
	swatch:SetVertexColor(r,g,b);
	frame.r, frame.g, frame.b = r,g,b;

	-- Update
	if k then
		if (k == "player") then
			SetPlayerVal("R", r, frame.CEPBVal);
			SetPlayerVal("G", g, frame.CEPBVal);
			SetPlayerVal("B", b, frame.CEPBVal);
			if (CEPB_PlayerCasting or CEPB_PlayerChanneling) then
				CEPlayerBar_ProcessBarSettingsWhileCasting();
			else
				CEPlayerBar_ShowXP();
				CEPlayerBar_RealUpdate();
			end
		elseif (k == "pet") then
			SetPetVal("R", r, frame.CEPBVal);
			SetPetVal("G", g, frame.CEPBVal);
			SetPetVal("B", b, frame.CEPBVal);
			if (CEPB_PetCasting or CEPB_PetChanneling) then
				CEPetBar_ProcessBarSettingsWhileCasting();
			else
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
			end
		end
	end
end

-------------------------------------------------

function CEPB:OptionsFrame_CancelColor(f,s,k,prev)
	-- Cancels the color selection
	local r,g,b = prev.r, prev.g, prev.b;
	local swatch = _G[s];
	local frame = _G[f];
	swatch:SetVertexColor(r, g, b);
	frame.r, frame.g, frame.b = r,g,b;
	
	-- Update
	if k then
		if (k == "player") then
			SetPlayerVal("R", r, frame.CEPBVal);
			SetPlayerVal("G", g, frame.CEPBVal);
			SetPlayerVal("B", b, frame.CEPBVal);
			if (CEPB_PlayerCasting or CEPB_PlayerChanneling) then
				CEPlayerBar_ProcessBarSettingsWhileCasting();
			else
				CEPlayerBar_ShowXP();
				CEPlayerBar_RealUpdate();
			end
		elseif (k == "pet") then
			SetPetVal("R", r, frame.CEPBVal);
			SetPetVal("G", g, frame.CEPBVal);
			SetPetVal("B", b, frame.CEPBVal);
			if (CEPB_PetCasting or CEPB_PetChanneling) then
				CEPetBar_ProcessBarSettingsWhileCasting();
			else
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
			end
		end
	end
end
